package com.sincere.morningcheck.controller;

import com.sincere.morningcheck.common.ServerResponse;
import com.sincere.morningcheck.model.ApiStudent;
import com.sincere.morningcheck.model.ApiStudentCheckReport;
import com.sincere.morningcheck.service.StudentCheckReportService;
import com.sincere.morningcheck.utils.ExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;


@RestController
@RequestMapping("check_report")
@Api(value = "CheckReportController",tags = "晨检机检查报告接口")
public class CheckReportController {


    @Autowired
    StudentCheckReportService studentCheckReportService;
    @ApiOperation(value = "根据学生的用户ID获取学生晨检报告")
    @ApiImplicitParam(name="stuUserId",value = "学生用户ID",required = true)
    @GetMapping("getCheckReport")
    public ServerResponse<ApiStudentCheckReport> getCheckReport(@RequestParam String stuUserId){

        return studentCheckReportService.getCheckReport(stuUserId);
    }

    @ApiOperation(value = "根据学校获取学生晨检报告")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "schoolId",value = "学校ID",required = true),
            @ApiImplicitParam(name = "checkTime",value = "检查报告的时间，格式为yyyy-MM-dd",required = true)
    })
    @GetMapping("getCheckReportBySchoolId")
    public ServerResponse getCheckReportBySchoolId(@RequestParam Integer schoolId,@RequestParam String checkTime){

        return studentCheckReportService.getCheckReportBySchool(schoolId,checkTime);
    }

    @ApiOperation(value = "根据年级获取学生异常晨检报告的比例")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "schoolId",value = "学校ID",required = true),
            @ApiImplicitParam(name = "gradeId",value = "年级标识",required = true)
    })
    @GetMapping("getCheckExReportByGrade")
    public ServerResponse getCheckExReportByGrade(@RequestParam Integer schoolId,@RequestParam Integer gradeId,@RequestParam String checkTime){

        return studentCheckReportService.getCheckExReportByGrade(schoolId,gradeId,checkTime);
    }

    @ApiOperation(value = "学生晨检报告报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "schoolId",value = "学校ID",required = true),
            @ApiImplicitParam(name = "checkTime",value = "检查报告的时间，格式为yyyy-MM-dd",required = true),
            @ApiImplicitParam(name = "gradeId",value = "年级ID",defaultValue = "0")
    })
    @GetMapping("getCheckReportData")
    public ServerResponse getCheckReportData(@RequestParam Integer schoolId,@RequestParam String checkTime,@RequestParam(name = "gradeId",required = false) Integer gradeId){

        return studentCheckReportService.getCheckReportData(schoolId,checkTime,gradeId);
    }


    @ApiOperation("导出报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "schoolId",value = "学校ID",required = true),
            @ApiImplicitParam(name = "checkTime",value = "检查报告的时间，格式为yyyy-MM-dd",required = true),
            @ApiImplicitParam(name = "gradeId",value = "年级ID",defaultValue = "0")
    })
    @GetMapping("excelCheckReportData")
    public void excelCheckReportData(HttpServletResponse response, @RequestParam Integer schoolId, @RequestParam String checkTime,@RequestParam(name = "gradeId",required = false) Integer gradeId){

        //查询出需要导出的数据
        List<ApiStudent> apiStudentList=studentCheckReportService.getCheckReportData(schoolId,checkTime,gradeId).getData();
        //创建报表数据头
        List<String> head = new ArrayList<>();
        head.add("姓名");
        head.add("班级");
        head.add("入园时间");
        head.add("手检");
        head.add("眼检");
        head.add("口腔");
        head.add("喉咙");
        head.add("体温");
        head.add("腮部");
        head.add("外表");
        head.add("外伤情况");
        head.add("指甲");
        head.add("牙齿");
        head.add("其他");
        head.add("处理情况");
        //创建报表体
        List<List<String>> body = new ArrayList<>();//模拟二维集合
        if(apiStudentList!=null) {
            for (ApiStudent stu : apiStudentList) {
                List<String> bodyValue = new ArrayList<>();
                bodyValue.add(String.valueOf(stu.getStuName()));
                bodyValue.add(stu.getClassName());
                bodyValue.add(String.valueOf(stu.getCheckTime()));
                if (stu.getCheckResultObj().getHandNoEx()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("异常cellStyle");
                }
                if (stu.getCheckResultObj().getNoRedEye()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("红眼cellStyle");
                }
                if (stu.getCheckResultObj().getMouthNoEx()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("异常cellStyle");
                }
                if (stu.getCheckResultObj().getNoPharyngitis()&&stu.getCheckResultObj().getNoCough()) {
                    bodyValue.add("正常");
                } else {
                    String desc = "";
                    if(!stu.getCheckResultObj().getNoPharyngitis()){
                        desc = desc + "咽炎、" ;
                    }
                    if(!stu.getCheckResultObj().getNoCough()){
                        desc = desc + "咳嗽" ;
                    }
                    bodyValue.add(desc+"cellStyle");
                }
                if (stu.getCheckResultObj().getNoFever()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("发烧cellStyle");
                }
                if (stu.getCheckResultObj().getNoCrotchBig()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("腮部肿大cellStyle");
                }
                if (stu.getCheckResultObj().getNoPoorSpirit()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("精神不佳cellStyle");
                }
                if (stu.getCheckResultObj().getNoTrauma()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("外伤cellStyle");
                }
                if (stu.getCheckResultObj().getNoNail()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("过长cellStyle");
                }
                if (stu.getCheckResultObj().getNoToothDecay()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("蛀牙cellStyle");
                }
                if (stu.getCheckResultObj().getOther()) {
                    bodyValue.add("正常");
                } else {
                    bodyValue.add("异常cellStyle");
                }
                bodyValue.add("");
                //将数据添加到报表体中
                body.add(bodyValue);
            }
        }
        String fileName = checkTime+"_学生晨检报告.xls";
        HSSFWorkbook excel = ExcelUtils.expExcel(head,body);
        ExcelUtils.outFileStream(excel,fileName,response);
    }
}
