package com.sincere.morningcheck.controller;

import com.alibaba.fastjson.JSON;
import com.sincere.morningcheck.common.AESDeEncoder;
import com.sincere.morningcheck.common.MD5;
import com.sincere.morningcheck.common.ServerResponse;
import com.sincere.morningcheck.model.*;
import com.sincere.morningcheck.service.FileUpAndDownService;
import com.sincere.morningcheck.service.StudentCheckReportService;
import com.sincere.morningcheck.service.StudentService;
import com.sincere.morningcheck.service.UserServer;
import com.sincere.morningcheck.utils.ApiHelper;
import com.sincere.morningcheck.utils.DataConvertHelper;
import com.sincere.morningcheck.utils.LogUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("user")
@Api(value = "MorningCheckController", tags = "晨检机晨检接口说明")
public class MorningCheckController {


    @Value("${salt}")
    private String salt;

    @Autowired
    UserServer userServer;

    @Autowired
    StudentService studentService;

    @Autowired
    FileUpAndDownService fileUpAndDownService;

    @Autowired
    StudentCheckReportService studentCheckReportService;

    @ApiOperation("设备请求登陆接口")
    @ApiImplicitParams ({
            @ApiImplicitParam(name = "deviceNo", value = "设备号", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "password", value = "密码", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "authSign", value = "签名", dataType = "String",required = true,paramType = "query"),
    })
    @PostMapping("login")
    public ServerResponse login(@RequestParam String deviceNo, @RequestParam String password,@RequestParam String authSign){

        ServerResponse serverResponse = null;
        LogUtil.printInfoLog("进入登陆接口");
        Map map = new HashMap();
        map.put("password", password);
        map.put("deviceNo", deviceNo);

        String queryUrl = DataConvertHelper.mapAscSortToUrl(map);
        System.out.println(queryUrl);
        try {
            String signUrl = queryUrl.replace("&","");
//            String md5 = MD5.md5(signUrl, salt);
            boolean verify = MD5.verify(signUrl,salt,authSign);
            System.out.println("校验是否成功："+verify);
            if(verify) {

                User user = userServer.getDeviceUser(deviceNo);
                user.setParameter("0");
                String token = user.getDeviceId()+","+user.getSchoolId();//AESDeEncoder.AESEncode("token",user.getDeviceId()+","+user.getSchoolId());
                user.setToken(token);
                LogUtil.printInfoLog("生成的Token信息包括设备标识和学校标识"+token);
                serverResponse = ServerResponse.createBySuccess("success",user);

            }else{

                serverResponse = ServerResponse.createByErrorMessage("签名校验失败");
                LogUtil.printInfoLog("签名校验失败");

            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            serverResponse = ServerResponse.createByErrorMessage(ex.getMessage());
        }

        return serverResponse;
    }

    @ApiOperation("获取学生、卡号信息")
    @ApiImplicitParams ({
            @ApiImplicitParam(name = "token", value = "令牌", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "schoolId", value = "学校ID", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "authSign", value = "签名", dataType = "String",required = true,paramType = "query")
    })
    @PostMapping("getStudents")
    public ServerResponse getStudents(@RequestParam String token,@RequestParam Integer schoolId,@RequestParam String authSign){

        ServerResponse serverResponse = null;

        Map map = new HashMap();
        map.put("token", token);
        map.put("schoolId", String.valueOf(schoolId));

        String queryUrl = DataConvertHelper.mapAscSortToUrl(map);
        try {
            String signUrl = queryUrl.replace("&","");
            //验证签名，看签名是否正确
            boolean verify = MD5.verify(signUrl,salt,authSign);
//            boolean verify = true;
            System.out.println("签名是否正确"+verify);
            if(verify) {

                School school = userServer.getSchoolById(schoolId);
                serverResponse = ServerResponse.createBySuccess("success",school);
            }
            else{

                serverResponse = ServerResponse.createByErrorMessage("签名校验失败");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            serverResponse = ServerResponse.createByErrorMessage(ex.getMessage());
        }

        return serverResponse;
    }


    @ApiOperation("刷卡签到")
    @ApiImplicitParams ({
            @ApiImplicitParam(name = "token", value = "令牌", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "cardNo", value = "十进制卡号", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "imageId", value = "接送图像类型对应的photoId", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "authSign", value = "签名", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "date", value = "晨检日期(上传历史签到数据才有这个参数，不是当天的签到)\n" +
                    "时间格式为 : yyyy-MM-dd-HH-mm-ss", dataType = "String",paramType = "query")
    })
    @PostMapping("signByCard")
    public ServerResponse swipeCard(@RequestParam String token,@RequestParam String cardNo,@RequestParam Integer imageId,@RequestParam String authSign,@RequestParam(required = false) String date){

        LogUtil.printInfoLog("刷卡签到传入的参数：卡号"+cardNo+",图片ID："+imageId+",签名"+authSign);
        ServerResponse serverResponse = null;

        String card = DataConvertHelper.distToX(Long.valueOf(cardNo));

        Map<String,String> map = new HashMap();
        map.put("token", token);
        map.put("cardNo", cardNo);//传入的是10进制的卡号
        map.put("imageId",String.valueOf(imageId));

        Date dateIn = null;
        if(StringUtils.isBlank(date)){
            Calendar calendar= Calendar.getInstance();
            dateIn= calendar.getTime();

        }else{
            map.put("date",date);//当date存在的时候再传
            try {
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
                dateIn = dateFormat.parse(date);
                System.out.println(date);//2019-07-30-10-43-41
            }catch (ParseException px){
                px.printStackTrace();
            }
        }
        String queryUrl = DataConvertHelper.mapAscSortToUrl(map);
        try {
            String signUrl = queryUrl.replace("&","");
//            boolean verify = true;
            boolean verify = MD5.verify(signUrl,salt,authSign);
            System.out.println("签名是否正确"+verify);
            if(verify) {

                LogUtil.printInfoLog(token);;
                String deviceId = token.split(",")[0];
                Map<String,Object> hashMap = new HashMap<>();
                //设备编号
                hashMap.put("att_id",deviceId);
                hashMap.put("card_num",card);
                hashMap.put("func_no","08");
                hashMap.put("flag",0);
                hashMap.put("intime",dateIn);
                System.out.println("设备标识"+deviceId);

                studentService.swipeCard(hashMap);
                System.out.println("集合数量："+hashMap.size());
                System.out.println(hashMap.get("arr"));
                System.out.println(hashMap.get("iscuccess"));

                //message: 学生姓名你好
                Student student = studentService.getStudentByCardNo(card);
                String msg = String.format("学生%s你好",student.getStuName());
                serverResponse = ServerResponse.createBySuccessMessage(msg);
            } else{

                serverResponse = ServerResponse.createByErrorMessage("签名校验失败");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            serverResponse = ServerResponse.createByErrorMessage(ex.getMessage());
        }

        return serverResponse;
    }


    @ApiOperation("离园签到")
    @ApiImplicitParams ({
            @ApiImplicitParam(name = "token", value = "令牌", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "cardNo", value = "卡号", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "imageId", value = "接送图像类型对应的photoId", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "authSign", value = "签名", dataType = "String",required = true,paramType = "query")
    })
    @PostMapping("signOutByCard")
    public ServerResponse signOutByCard(@RequestParam String token,@RequestParam String cardNo,@RequestParam Integer imageId,@RequestParam String authSign){

        ServerResponse serverResponse = null;
        LogUtil.printInfoLog("离园签到传入的参数：卡号"+cardNo+",图片ID："+imageId+",签名"+authSign);

        String card = DataConvertHelper.distToX(Long.valueOf(cardNo));

        Map<String,String> map = new HashMap();
        map.put("token", token);
        map.put("cardNo", cardNo);//传入的是10进制的卡号
        map.put("imageId",String.valueOf(imageId));

        String queryUrl = DataConvertHelper.mapAscSortToUrl(map);
        try {
            String signUrl = queryUrl.replace("&","");
            boolean verify = MD5.verify(signUrl,salt,authSign);
            System.out.println("签名是否正确"+verify);
            if(verify) {
                String deviceId = token.split(",")[0];
                Map<String,Object> hashMap = new HashMap<>();
                hashMap.put("att_id",deviceId);
                hashMap.put("card_num",card);
                hashMap.put("func_no","08");
                hashMap.put("flag",0);
                hashMap.put("intime",new Date());
                studentService.swipeCard(hashMap);
                System.out.println(hashMap.get("arr"));
                System.out.println(hashMap.get("iscuccess"));
                //message: 学生姓名你好
                Student student = studentService.getStudentByCardNo(card);
                String msg = String.format("学生%s你好",student.getStuName());
                serverResponse = ServerResponse.createBySuccessMessage(msg);
            }else{
                serverResponse = ServerResponse.createByErrorMessage("签名校验失败");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            serverResponse = ServerResponse.createByErrorMessage(ex.getMessage());
        }

        return serverResponse;
    }
    @RequestMapping(value = "/uploadImg", method = RequestMethod.POST)
    @ApiOperation("上传多个文件")
    @ApiImplicitParams ({
            @ApiImplicitParam(name = "token", value = "令牌", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "file", value = "文件列表", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "type", value = "图像类型", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "authSign", value = "签名", dataType = "String",required = true,paramType = "query")
    })
    public ServerResponse moreFileUpload(@RequestParam(value = "file", required = false) MultipartFile[] file, @RequestParam(value = "token") String token, @RequestParam(value="type") String type,@RequestParam String authSign) {

        LogUtil.printInfoLog("上传多个文件传入的类型："+type+",签名："+authSign);

        ServerResponse serverResponse = null;
        Map map = new HashMap();
        map.put("token", token);
        map.put("type", type);

        String[] fileNames = type.split("_");
        if(fileNames.length != file.length){
            return ServerResponse.createByErrorMessage("类型数量和文件数量不一致，程序中断");
        }

        String queryUrl = DataConvertHelper.mapAscSortToUrl(map);
        try {
            String signUrl = queryUrl.replace("&","");
//            String md5 = MD5.md5(signUrl, salt);
            boolean verify = MD5.verify(signUrl,salt,authSign);
            if(verify) {
                try {
                    serverResponse = fileUpAndDownService.uploadPicture(file,fileNames);
                } catch (Exception e) {
                    serverResponse = ServerResponse.createByErrorMessage(String.format(">>>>>>图片上传异常，e=%s", e.getMessage()));
                }
            }else{
                serverResponse = ServerResponse.createByErrorMessage("签名校验失败");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            serverResponse = ServerResponse.createByErrorMessage(ex.getMessage());
        }

        return serverResponse;
    }


    @ApiOperation("晨检报告")
    @ApiImplicitParams ({
            @ApiImplicitParam(name = "token", value = "令牌", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "cardNo", value = "卡号", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name = "result", value = "检查结果,32位字符串", dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name="robotResult",value = "晨检机器判断结果。3位字符串",dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name="access",value = "入园许可。 Y : 入园   N：离园",dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name="temperature",value = "体温（小于35.2时显示LOW；大于42.2时，显示HIGH）",dataType = "String",required = true,paramType = "query"),
            @ApiImplicitParam(name="handImgId",value = "手部异常图像ID",dataType = "String",paramType = "query"),
            @ApiImplicitParam(name = "mouthImgId", value = "嘴部异常头像ID", dataType = "String",paramType = "query"),
            @ApiImplicitParam(name = "eyeImgId", value = "眼部异常头像ID", dataType = "String",paramType = "query"),
            @ApiImplicitParam(name = "authSign", value = "签名", dataType = "String",required = true,paramType = "query"),
    })
    @PostMapping("uploadCheckReport")
    public ServerResponse uploadCheckReport(@RequestParam String token,@RequestParam String cardNo,@RequestParam String result,@RequestParam String robotResult,@RequestParam String access,@RequestParam String temperature,@RequestParam(required = false) String handImgId,@RequestParam(required = false) String mouthImgId,@RequestParam(required = false) String eyeImgId,@RequestParam String authSign){

        LogUtil.printInfoLog("传入的参数：卡号"+cardNo+",检查结果："+result+"，检查机判断结果："+robotResult+",签名"+authSign+"入园许可。 Y : 入园   N：离园:入园情况："+access);

        ServerResponse serverResponse;

        String card = DataConvertHelper.distToX(Long.valueOf(cardNo));

        Map<String,String> map = new HashMap();
        map.put("token", token);
        map.put("cardNo", cardNo);//传入的是10进制的卡号
        map.put("result",result);
        map.put("robotResult",robotResult);
        map.put("access",access);
        map.put("temperature",temperature);
        map.put("handImgId",handImgId);
        map.put("mouthImgId",mouthImgId);
        map.put("eyeImgId",eyeImgId);
        Date checkDate = new Date();
        String queryUrl = DataConvertHelper.mapAscSortToUrl(map);
        try {
            String signUrl = queryUrl.replace("&","");
            boolean verify = MD5.verify(signUrl,salt,authSign);
            if(verify) {
                //微信模板消息推送
                Thread  thread = new Thread(()->{

                    try {

                        String schoolId = token.split(",")[1];//AESDeEncoder.AESDncode("token",token).split(",")[1];
                        String stuUserId = studentService.getStuUserId(card, schoolId);
                        List<String> openIds = studentService.getOpenId(stuUserId, schoolId);
                        if (openIds != null) {

                            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

                            for (String openId : openIds) {
                                WXCheckTemplate wxCheckTemplate = new WXCheckTemplate();
                                wxCheckTemplate.setFirstData("家长您好，有一份晨检报告待查看");
                                wxCheckTemplate.setKeyword1("晨检");
                                wxCheckTemplate.setKeyword2(simpleDateFormat.format(checkDate));
                                wxCheckTemplate.setOpenID(openId);
                                wxCheckTemplate.setSchoolID(schoolId);
                                wxCheckTemplate.setUrl("http://campus.myjxt.com/Web/Skin/EasyNotice/Modules/EMGzh/XSTVerification3.html?type=5");
                                String jsonResult = JSON.toJSONString(wxCheckTemplate);
                                LogUtil.printInfoLog("微信模板接口调用："+jsonResult);
                                String returnResult = ApiHelper.doPost("http://campus.myjxt.com/api/SendGzhTemplate/SengMorningCheck", null, jsonResult);
                            }
                        }
                    }catch (Exception ex){
                        LogUtil.printInfoLog("异常信息"+ex.getMessage());
                    }

                });
                thread.start();

                StudentCheckReport studentCheckReport = new StudentCheckReport();
                studentCheckReport.setAccess(access);
                studentCheckReport.setCardNo(card);
                studentCheckReport.setCheckResult(result);
                studentCheckReport.setRobotResult(robotResult);

                studentCheckReport.setCheckTime(checkDate);
                studentCheckReport.setInTime(checkDate);
                studentCheckReport.setTemperature(temperature);
                studentCheckReport.setEyeImgId(eyeImgId);
                studentCheckReport.setHandImgId(handImgId);
                studentCheckReport.setMouthImgId(mouthImgId);

                //message: 学生姓名你好
                Student student = studentService.getStudentByCardNo(card);
                String msg = String.format("学生%s你好，你的晨检报告已经自动生成",student.getStuName());
                studentCheckReport.setStudent_id(student.getStuId());

                StudentCheckReport studentCheckReportEntity= studentCheckReportService.getCheckReport(student.getStuId());
                if(studentCheckReportEntity==null){
                    studentCheckReportService.insert(studentCheckReport);
                }else{
                    studentCheckReport.setId(studentCheckReportEntity.getId());
                    int updateCount = studentCheckReportService.updateByStudent(studentCheckReport);
                }

                StudentCheckReportHis studentCheckReportHis = new StudentCheckReportHis();
                studentCheckReportHis.setAccess(studentCheckReport.getAccess());
                studentCheckReportHis.setCardNo(studentCheckReport.getCardNo());
                studentCheckReportHis.setCheck_id(studentCheckReport.getId());
                studentCheckReportHis.setCheckResult(studentCheckReport.getCheckResult());
                studentCheckReportHis.setRobotResult(studentCheckReport.getRobotResult());
                studentCheckReportHis.setInTime(studentCheckReport.getInTime());
                studentCheckReportHis.setCheckTime(studentCheckReport.getCheckTime());
                studentCheckReportHis.setTemperature(studentCheckReport.getTemperature());
                studentCheckReportHis.setEyeImgId(studentCheckReport.getEyeImgId());
                studentCheckReportHis.setHandImgId(studentCheckReport.getHandImgId());
                studentCheckReportHis.setMouthImgId(studentCheckReport.getMouthImgId());
                studentCheckReportHis.setStudent_id(studentCheckReport.getStudent_id());
                studentCheckReportService.insertCheckHis(studentCheckReportHis);
                serverResponse = ServerResponse.createBySuccessMessage(msg);
            }else{

                serverResponse = ServerResponse.createByErrorMessage("签名校验失败");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            serverResponse = ServerResponse.createByErrorMessage(ex.getMessage());
        }

        return serverResponse;
    }
}
