package com.sincere.morningcheck.service.impl;

import com.sincere.morningcheck.common.ImgType;
import com.sincere.morningcheck.common.ServerResponse;
import com.sincere.morningcheck.config.MessageProperties;
import com.sincere.morningcheck.dao.FileDao;
import com.sincere.morningcheck.model.FileEntity;
import com.sincere.morningcheck.model.Photo;
import com.sincere.morningcheck.service.FileUpAndDownService;
import net.coobird.thumbnailator.Thumbnails;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class FileUpAndDownServiceImpl implements FileUpAndDownService {
    @Autowired
    private MessageProperties config; //用来获取file-message.properties配置文件中的信息

    @Autowired
    private FileDao fileDao;

    @Override
    public ServerResponse uploadPicture(MultipartFile[] files, String[] file_types) throws Exception {

        List<FileEntity> fileEntities = new LinkedList<>();
        int i = 0;
        for(MultipartFile file : files) {
            System.out.println(file.getOriginalFilename());

            try {

                Map<String, Object> resMap = new HashMap<>();
                String[] IMAGE_TYPE = config.getImageType().split(",");
                String path = null;
                boolean flag = false;
                for (String type : IMAGE_TYPE) {
                    if (StringUtils.endsWithIgnoreCase(file.getOriginalFilename(), type)) {
                        flag = true;
                        break;
                    }
                }
                if (flag) {

                    // 获得文件类型
                    String fileType = file.getContentType();
                    // 原名称
                    String oldFileName = file.getOriginalFilename();
                    // 获得文件后缀名称
                    String suffix = oldFileName.substring(oldFileName.lastIndexOf("."));//带逗号，不带逗号加1
                    // 新名称
                    String imgType = file_types[i];
                    UUID uuid = UUID.randomUUID();
                    String newFileName = imgType+ "_" + uuid.toString().replace("-","") +suffix;
                    i++;
                    // 年月日文件夹
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                    String basedir = sdf.format(new Date());
                    String rePath = "/images/" + basedir + "/" + newFileName;
                    // 进行压缩(大于4M)
                    if (file.getSize() > config.getFileSize()) {
                        path = config.getUpPath() + "/" + basedir + "/" + newFileName;
                        // 如果目录不存在则创建目录
                        File oldFile = new File(path);
                        if (!oldFile.exists()) {
                            oldFile.mkdirs();
                        }
                        file.transferTo(oldFile);
                        // 压缩图片
                        Thumbnails.of(oldFile).scale(config.getScaleRatio()).toFile(path);

                    } else {
                        path = config.getUpPath() + "/" + basedir + "/" + newFileName;
                        // 如果目录不存在则创建目录
                        File uploadFile = new File(path);
                        if (!uploadFile.exists()) {
                            uploadFile.mkdirs();
                        }
                        file.transferTo(uploadFile);
                    }

                    FileEntity fileEntity = new FileEntity();

                    fileEntity.setFileName(newFileName);
                    fileEntity.setImgRePath(rePath);
                    fileEntity.setOldFileName(oldFileName);
                    fileEntity.setImgType(ImgType.valueOf(imgType));

                    fileEntity.setInTime( new java.sql.Date(new Date().getTime()));
                    fileEntity.setPhotoId(0);
                    insert(fileEntity);
                    fileEntities.add(fileEntity);

                } else {
                    return ServerResponse.createByErrorMessage("result:图片格式不正确,支持png|jpg|jpeg");
                }
            } catch (Exception e) {
                e.printStackTrace();
                throw new Exception(e.getMessage());
            }
        }

        //返回结果
        Map<String,Object> map = new HashMap<>();

        List<Photo> photoIds = new ArrayList<>();

        for (FileEntity fileEntity : fileEntities){
            Photo photo = new Photo();
            photo.setPhotoId(fileEntity.getPhotoId());
            photoIds.add(photo);
        }
        map.put("photoIds",photoIds);
        return ServerResponse.createBySuccess(map);
    }

    @Override
    public ServerResponse<Integer> insert(FileEntity fileEntity) {

        int result = fileDao.insert(fileEntity);
        return ServerResponse.createBySuccess(result);
    }

}
