package com.sincere.morningcheck.service.impl;

import com.sincere.morningcheck.common.ServerResponse;
import com.sincere.morningcheck.dao.StudentCheckReportDao;
import com.sincere.morningcheck.dao.StudentDao;
import com.sincere.morningcheck.model.*;
import com.sincere.morningcheck.service.StudentCheckReportService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class StudentCheckReportServiceImpl implements StudentCheckReportService {

    @Autowired
    StudentCheckReportDao studentCheckReportDao;

    @Autowired
    StudentDao studentDao;

    @Override
    public int insert(StudentCheckReport studentCheckReport) {
        return studentCheckReportDao.insert(studentCheckReport);
    }

    @Override
    public int insertCheckHis(StudentCheckReportHis studentCheckReportHis) {
        return studentCheckReportDao.insertCheckHis(studentCheckReportHis);
    }

    @Override
    public int updateByStudent(StudentCheckReport studentCheckReport) {
        return studentCheckReportDao.updateByStudent(studentCheckReport);
    }

    @Override
    public StudentCheckReport getCheckReport(Integer studentId) {
        List<StudentCheckReport> studentCheckReports = studentCheckReportDao.getCheckReport(studentId);
        if(studentCheckReports!=null&&studentCheckReports.size()>0){
            return studentCheckReports.get(0);
        }
        return null;
    }

    @Override
    public ServerResponse<ApiStudentCheckReport> getCheckReport(String stuUserId) {

        List<StudentCheckReport> studentCheckReports = new ArrayList<>();
        StudentCheckReport studentCheckReport= new StudentCheckReport();

        Student student = studentDao.getStudentByStuUserId(stuUserId);
        studentCheckReports = studentCheckReportDao.getCheckReport(student.getStuId());
        Boolean isMorningCheck;
        if(studentCheckReports!=null&&studentCheckReports.size()>0){

                studentCheckReport = studentCheckReports.get(0);
                isMorningCheck = true;

        }else{
            isMorningCheck = false;
        }

        ApiStudentCheckReport apiStudentCheckReport = getApiStudentReport(studentCheckReport,isMorningCheck);
        return ServerResponse.createBySuccess(apiStudentCheckReport);
    }

    @Override
    public ServerResponse getCheckReportBySchool(Integer schoolId, String checkTime) {

        Map<String,Object> map = new HashMap<>();

        int studentAllCount = getStuCountBySchoolId(schoolId);
        int stuCheckStuCount = studentCheckReportDao.getCheckStuCountBySchoolId(schoolId,checkTime);
        map.put("studentAllCount",studentAllCount);
        map.put("stuCheckStuCount",stuCheckStuCount);

        List<StudentCheckReport> studentCheckReports = studentCheckReportDao.getCheckReportBySchoolId(schoolId,checkTime);

        List<StudentCheckReport> studentExCheckReports = studentCheckReports.stream().filter(n->n.getCheckResult().contains("N")).collect(Collectors.toList());
        //检查报告总人数
        int checkAllCount = studentCheckReports.size();
        //异常人数
        long exCheckCount = studentExCheckReports.size();
        //检查报告中正常的人数
        long checkCount  = checkAllCount - exCheckCount;

        map.put("checkCount",checkCount);
        map.put("exCheckCount",exCheckCount);

        List<Grade> grades= studentDao.getGradeBySchoolId(schoolId);

        map.put("grades",grades);

        List<String> checkTypeList = new ArrayList<>();
        List<Long> checkTypeCount = new ArrayList<>();
//        String[] checkTypeArrays ={"handNoEx","mouthNoEx","noRedEye","noFever","noCrotchBig","noPoorSpirit","noPharyngitis","noTrauma","noCough","noNail","noToothDecay","other"};

        List<CheckResult> checkResults = new ArrayList<>();
        for(StudentCheckReport checkReport:studentExCheckReports){
            CheckResult checkResult = new CheckResult();
            char[] chs = checkReport.getCheckResult().toCharArray();
            checkResult = getCheckResult(chs);
            checkResults.add(checkResult);
        }

        long count1 = checkResults.stream().filter(n->!n.getHandNoEx()).count();
        if(count1>0){
            checkTypeList.add("手部异常");
            checkTypeCount.add(count1);
        }
        long count2= checkResults.stream().filter(n->!n.getMouthNoEx()).count();
        if(count2>0){
            checkTypeList.add("口腔异常");
            checkTypeCount.add(count2);
        }
        long count3= checkResults.stream().filter(n->!n.getNoRedEye()).count();
        if(count3>0){
            checkTypeList.add("红眼");
            checkTypeCount.add(count3);
        }
        long count4= checkResults.stream().filter(n->!n.getNoFever()).count();
        if(count4>0){
            checkTypeList.add("发烧");
            checkTypeCount.add(count4);
        }

        long count5= checkResults.stream().filter(n->!n.getNoCrotchBig()).count();
        if(count5>0){
            checkTypeList.add("腮部肿大");
            checkTypeCount.add(count5);
        }

        long count6= checkResults.stream().filter(n->!n.getNoPoorSpirit()).count();
        if(count6>0){
            checkTypeList.add("精神不佳");
            checkTypeCount.add(count6);
        }
        long count7= checkResults.stream().filter(n->!n.getNoPharyngitis()).count();
        if(count7>0){
            checkTypeList.add("咽炎");
            checkTypeCount.add(count7);
        }

        long count8= checkResults.stream().filter(n->!n.getNoTrauma()).count();
        if(count8>0){
            checkTypeList.add("外伤");
            checkTypeCount.add(count8);
        }

        long count9= checkResults.stream().filter(n->!n.getNoCough()).count();
        if(count9>0){
            checkTypeList.add("咳嗽");
            checkTypeCount.add(count9);
        }

        long count10= checkResults.stream().filter(n->!n.getNoNail()).count();
        if(count10>0){
            checkTypeList.add("指甲过长");
            checkTypeCount.add(count10);
        }

        long count11= checkResults.stream().filter(n->!n.getNoToothDecay()).count();
        if(count11>0){
            checkTypeList.add("蛀牙");
            checkTypeCount.add(count11);
        }

        long count12= checkResults.stream().filter(n->!n.getOther()).count();
        if(count12>0){
            checkTypeList.add("其他");
            checkTypeCount.add(count12);
        }
        map.put("checkTypeList",checkTypeList);
        map.put("checkTypeCount",checkTypeCount);
        return ServerResponse.createBySuccess(map);
    }

    @Override
    public ServerResponse getCheckExReportByGrade(Integer schoolId, Integer gradeId,String checkTime) {

        List<ClassExReport> classExReports = studentCheckReportDao.getCheckExReportByGrade(schoolId,gradeId,checkTime);
        float count = 0;
        for(ClassExReport classExReport : classExReports){
            count = count + classExReport.getCount();
        }

        if(count>0){
            for(ClassExReport classExReport : classExReports){
               classExReport.setValue((classExReport.getCount()/count)*100);
            }
        }else {
            return ServerResponse.createByErrorCodeMessage(10,"暂无数据");
        }
        return ServerResponse.createBySuccess(classExReports);
    }

    @Override
    public ServerResponse<List<ApiStudent>> getCheckReportData(Integer schoolId, String checkTime,int gradeId) {
        List<StudentCheckReport> studentCheckReports = studentCheckReportDao.getCheckReportBySchoolId(schoolId,checkTime);

        List<ApiStudent> apiStudents = new ArrayList<>();
        for(StudentCheckReport item : studentCheckReports){
            ApiStudent apiStudent = new ApiStudent();
            apiStudent.setCardNo(item.getCardNo());
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm");
            apiStudent.setCheckTime(simpleDateFormat.format(item.getCheckTime()));
            apiStudent.setTemperature(item.getTemperature());
            if(StringUtils.isNotBlank(item.getCheckResult())){
                char[] chs = item.getCheckResult().toCharArray();
                apiStudent.setCheckResultObj(getCheckResult(chs));
            }
            if(StringUtils.isNotBlank(item.getCardNo())){
                 Student student = studentDao.getStudentByStudentId(schoolId,item.getStudent_id());
                 apiStudent.setClassName(student.getClassName());
                 apiStudent.setStudent_id(student.getStuId());
                 apiStudent.setStuName(student.getStuName());
                 apiStudent.setClassId(student.getClassId());
            }
            apiStudents.add(apiStudent);
        }
        if(gradeId>0) {
            apiStudents = apiStudents.stream().filter(n -> n.getClassId() == gradeId).collect(Collectors.toList());
        }
        if(apiStudents.size()<=0){
            return ServerResponse.createByErrorCodeMessage(10,"暂无数据");
        }
        return ServerResponse.createBySuccess(apiStudents);
    }

    private ApiStudentCheckReport getApiStudentReport(StudentCheckReport studentCheckReport,Boolean isMorningCheck){

        ApiStudentCheckReport apiStudentCheckReport = new ApiStudentCheckReport();
        if(isMorningCheck) {
            apiStudentCheckReport.setCardNo(studentCheckReport.getCardNo());
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            System.out.println(studentCheckReport.getCheckTime());
            apiStudentCheckReport.setCheckTime(simpleDateFormat.format(studentCheckReport.getCheckTime()));
            apiStudentCheckReport.setStudent_id(studentCheckReport.getStudent_id());
            apiStudentCheckReport.setTemperature(studentCheckReport.getTemperature());
            char[] chs = studentCheckReport.getCheckResult().toCharArray();
            CheckResult checkResult = getCheckResult(chs);
            apiStudentCheckReport.setCheckResultObj(checkResult);
        }
        apiStudentCheckReport.setIsMorningCheck(isMorningCheck);
        return apiStudentCheckReport;
    }

    private int getStuCountBySchoolId(Integer schoolId){

       return studentDao.getStuCountBySchoolId(schoolId);
    }

    /**
     * 解析字符串
     * @param chs
     * @return
     */
    private  CheckResult getCheckResult(char[] chs){
        CheckResult checkResult = new CheckResult();

        //前12位
        checkResult.setHandNoEx(isJudge(Character.toString(chs[0])));
        checkResult.setMouthNoEx(isJudge(Character.toString(chs[1])));
        checkResult.setNoRedEye(isJudge(Character.toString(chs[2])));
        checkResult.setNoFever(isJudge(Character.toString(chs[3])));
        checkResult.setNoCrotchBig(isJudge(Character.toString(chs[4])));
        checkResult.setNoPoorSpirit(isJudge(Character.toString(chs[5])));
        checkResult.setNoPharyngitis(isJudge(Character.toString(chs[6])));
        checkResult.setNoTrauma(isJudge(Character.toString(chs[7])));
        checkResult.setNoCough(isJudge(Character.toString(chs[8])));
        checkResult.setNoNail(isJudge(Character.toString(chs[9])));
        checkResult.setNoToothDecay(isJudge(Character.toString(chs[10])));
        checkResult.setOther(isJudge(Character.toString(chs[11])));

        return checkResult;
    }
    private Boolean isJudge(String yesOrNo){

        if(yesOrNo.equals("Y")){
            return true;
        }
        return false;
    }

}
