package com.sincre.springboot.common;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;


import java.io.Serializable;

/**
 * Created by Ziv
 */
@JsonInclude(JsonInclude.Include.NON_EMPTY)
@ApiModel
public class ServerResponse<T> implements Serializable {

    /**
     * 分页时用到的总数量
     */
    @ApiModelProperty(name="total",value = "分页时用到的总数量,当返回的是集合对象的时候，才有该值，否者该值不存在")
    private String total;
    /**
     * 操作状态码
     */
    @ApiModelProperty(name="code",value = "操作状态码")
    private int code;
    /**
     * 状态信息
     */
    @ApiModelProperty(name="msg",value = "状态信息")
    private String msg;
    /**
     * 返回的数据包
     */
    @ApiModelProperty(name="data",value = "返回的数据包")
    private T data;

    private ServerResponse(int code) {
        this.code = code;
    }

    private ServerResponse(int code, T data) {
        this.code = code;
        this.data = data;
    }

    private ServerResponse(int code, String msg, T data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }

    private ServerResponse(String total,int code, String msg, T data) {
        this.total = total;
        this.code = code;
        this.msg = msg;
        this.data = data;
    }
    private ServerResponse(int code, String msg) {
        this.code = code;
        this.msg = msg;
    }

    @JsonIgnore
    //使之不在json序列化结果当中
    public boolean isSuccess() {
        return this.code == ResponseCode.SUCCESS.getCode();
    }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    public String getTotal() {
        return total;
    }
    public int getCode() {
        return code;
    }

    public T getData() {
        return data;
    }

    public String getMsg() {
        return msg;
    }


    public static <T> ServerResponse<T> createBySuccess() {
        return new ServerResponse<T>(ResponseCode.SUCCESS.getCode());
    }

    /*
       该方法对应的是private ServerResponse(int code,String msg)的构造方法
     */
    public static <T> ServerResponse<T> createBySuccessMessage(String msg) {
        return new ServerResponse<T>(ResponseCode.SUCCESS.getCode(), msg);
    }

    /*
        该方法对应的是private ServerResponse(int code,T data)构造方法
     */
    public static <T> ServerResponse<T> createBySuccess(T data) {
        return new ServerResponse<T>(ResponseCode.SUCCESS.getCode(), data);
    }

    public static <T> ServerResponse<T> createBySuccess(String msg, T data) {
        return new ServerResponse<T>(ResponseCode.SUCCESS.getCode(), msg, data);
    }

    public static <T> ServerResponse<T> createBySuccess(String total,String msg, T data) {
        return new ServerResponse<T>(total,ResponseCode.SUCCESS.getCode(), msg, data);
    }


    public static <T> ServerResponse<T> createByError() {
        return new ServerResponse<T>(ResponseCode.ERROR.getCode(), ResponseCode.ERROR.getDesc());
    }


    public static <T> ServerResponse<T> createByErrorMessage(String errorMessage) {
        return new ServerResponse<T>(ResponseCode.ERROR.getCode(), errorMessage);
    }

    public static <T> ServerResponse<T> createByErrorCodeMessage(int errorCode, String errorMessage) {
        return new ServerResponse<T>(errorCode, errorMessage);
    }

}
