package com.sincre.springboot.controller;

import com.alibaba.fastjson.JSON;
import com.sincre.springboot.ApiModel.FacePlusResult;
import com.sincre.springboot.ApiPlatform.FacePlusConfig;
import com.sincre.springboot.common.ServerResponse;
import com.sincre.springboot.utils.ApiHelper;
import com.sincre.springboot.utils.ResultUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.omg.CORBA.Any;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.spring.web.json.Json;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(value = "/FacePlus/*")
@Api(tags = "旷视云服务API")
public class FacePlusController {


    @ApiOperation("人脸数据注册")
    @ApiImplicitParam(name = "imageUrl",value = "人脸图片地址",required = true)
    @PostMapping("detect")
    public ServerResponse detect(String imageUrl){

        String url = FacePlusConfig.hostUrl + "/facepp/v3/detect";
        Map<String,String> head = new HashMap<>();
        Map<String,Object> body = setUpCommonBody();
        body.put("image_url",imageUrl);

        String result = ApiHelper.doPost(url,head,body);
        FacePlusResult faceResult = JSON.parseObject(result,FacePlusResult.class);
        return ResultUtils.getInstance().returnResult_Face(faceResult,result);
    }


    @ApiOperation("人脸数据比较")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "faceToken1",value = "人脸1token",required = true),
            @ApiImplicitParam(name = "faceToken2",value = "人脸2token",required = true)
    })
    @PostMapping("compare")
    public ServerResponse compare(String faceToken1,String faceToken2){

        String url = FacePlusConfig.hostUrl + "/facepp/v3/compare";
        Map<String,String> head = new HashMap<>();
        Map<String,Object> body = setUpCommonBody();
        body.put("face_token1",faceToken1);
        body.put("face_token2",faceToken2);

        String result = ApiHelper.doPost(url,head,body);
        FacePlusResult faceResult = JSON.parseObject(result,FacePlusResult.class);
        return ResultUtils.getInstance().returnResult_Face(faceResult,result);
    }

    @ApiOperation("人脸数据查询")
    @ApiImplicitParam(name = "faceToken",value = "人脸token",required = true)
    @PostMapping("search")
    public ServerResponse search(String faceToken){

        String url = FacePlusConfig.hostUrl + "/facepp/v3/search";
        Map<String,String> head = new HashMap<>();
        Map<String,Object> body = setUpCommonBody();
        body.put("face_token",faceToken);

        String result = ApiHelper.doPost(url,head,body);
        FacePlusResult faceResult = JSON.parseObject(result,FacePlusResult.class);
        return ResultUtils.getInstance().returnResult_Face(faceResult,result);
    }

    /**
     * 通用参数
     * @return
     */
    private Map<String,Object> setUpCommonBody(){

        Map<String,Object> map = new HashMap<>();
        map.put("api_key", FacePlusConfig.apiKey);
        map.put("api_secret",FacePlusConfig.apiSecret);
        return map;
    }
}
