package com.sincre.springboot.controller;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sincre.springboot.ApiModel.*;
import com.sincre.springboot.ApiPlatform.TuYaCloudService;
import com.sincre.springboot.common.ServerResponse;
import com.sincre.springboot.utils.ApiHelper;
import com.sincre.springboot.utils.CacheHelper;
import com.sincre.springboot.utils.ResultUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.spring.web.json.Json;

import javax.annotation.security.PermitAll;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(value = "/TuYa/*")
@Api(tags = "涂鸦云服务API调用")
public class TuYaYunController {

    @ApiOperation("根据设备ID来获取设备的信息")
    @ApiImplicitParam(name="deviceId",value = "设备ID",required = true)
    @GetMapping("getDeviceInfoByDeviceID")
    public ServerResponse getDeviceInfoByDeviceID(String deviceId){

        String apiUrl = String.format("/v1.0/devices/%s",deviceId);
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("批量获取设备信息")
    @ApiImplicitParam(name="deviceIds",value = "设备ID 列表，多个设备号用逗号隔开",required = true)
    @GetMapping("GetDevicesInfo")
    public ServerResponse getTuYaDevicesInfo(String deviceIds){

        String apiUrl = "/v1.0/devices?device_ids="+deviceIds;
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("获取设备状态信息")
    @ApiImplicitParam(name="deviceId",value = "设备ID",required = true)
    @GetMapping("getStatusByDeviceID")
    public ServerResponse getStatusByDeviceID(String deviceId){

        String apiUrl = String.format("/v1.0/devices/%s/status",deviceId);
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }


    @ApiOperation("批量获取设备状态")
    @ApiImplicitParam(name="deviceIds",value = "设备ID 列表，多个设备号用逗号隔开",required = true)
    @GetMapping("getTuYaStatusInfo")
    public ServerResponse getTuYaStatusInfo(String deviceIds){

        String apiUrl = "/v1.0/devices/status?device_ids="+deviceIds;
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    /**
     * 根据schema分页获取开发者App下的用户列表
     * @param AppId  应用标识
     * @param pageIndex  页码
     * @param pageSize   大小
     * @return
     */
    @ApiOperation(value = "获取用户信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name="AppId",value = "应用标识",required = true),
            @ApiImplicitParam(name="pageIndex", value = "分页起始页，从1开始",required = true),
            @ApiImplicitParam(name="pageSize", value = "分页大小，默认为10，最大为50")
    })
    @GetMapping("{AppId}/GetTuYaAPPUser")
    public ServerResponse getTuYaAPPUser(@PathVariable("AppId") String AppId, @RequestParam int pageIndex, @RequestParam(defaultValue = "10") int pageSize){

        String apiUrl = String.format("/v1.0/apps/%s/users?page_no=%d&page_size=%d",AppId,pageIndex,pageSize);//iotshunzhi
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        //Token获取后的签名
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> headMap = new HashMap();
        headMap.put("client_id", TuYaCloudService.ClientId);
        headMap.put("access_token",access_Token);
        headMap.put("sign",sign);
        headMap.put("sign_method",TuYaCloudService.Sign_method);
        headMap.put("t",t.toString());

        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,headMap);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }


    @ApiOperation("注册用户")
    @ApiImplicitParam(name="AppId",value = "应用标识",required = true)
    @PostMapping("{AppId}/AddTuYaAPPUser")
    public ServerResponse addTuYaAPPUser(@PathVariable("AppId") String AppId, @RequestBody UserInfo userInfo){

        String apiUrl = String.format("/v1.0/apps/%s/user",AppId);//应用标识iotshunzhi
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        //Token获取后的签名
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> headMap = new HashMap();
        headMap.put("client_id", TuYaCloudService.ClientId);
        headMap.put("access_token",access_Token);
        headMap.put("sign",sign);
        headMap.put("sign_method",TuYaCloudService.Sign_method);
        headMap.put("t",t.toString());

        ObjectMapper objectMapper = new ObjectMapper();
        String jsonResult = "";
        try {
            jsonResult = objectMapper.writeValueAsString(userInfo);
        }catch (com.fasterxml.jackson.core.JsonProcessingException ex){
            ex.printStackTrace();
        }

        String result = ApiHelper.doPost(TuYaCloudService.TuYaOpenUrl + apiUrl,headMap,jsonResult);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    /**
     * 根据用户ID来获取用户账号下的所有设备
     * @param uid
     * @return
     */
    @ApiOperation("根据用户ID来获取用户账号下的所有设备")
    @ApiImplicitParam(name="uid",value = "用户标识",required = true)
    @GetMapping("{uid}/GetDevicesByUid")
    public ServerResponse getDevicesByUid(@PathVariable("uid") String uid){

        String apiUrl = String.format("/v1.0/users/%s/devices",uid);
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        //Token获取后的签名
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());

        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        System.out.println(result);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }
    /**
     * 生成配网的Token
     * @return
     */
    @ApiOperation("生成配网的Token")
    @ApiImplicitParam(name="uid",value = "用户标识",required = true)
    @GetMapping("AddDevicesToken")
    public ServerResponse addDevicesToken(@RequestParam String uid){

        String apiUrl = String.format("/v1.0/devices/token");
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        //Token获取后的签名
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());

        ObjectMapper objectMapper = new ObjectMapper();
        DeviceToken deviceToken = new DeviceToken();
        deviceToken.setUid(uid);
        deviceToken.setTimeZoneId("Asia/ZheJiang");

        String jsonResult = "";
        try {
            jsonResult = objectMapper.writeValueAsString(deviceToken);
        }catch (com.fasterxml.jackson.core.JsonProcessingException ex){
            ex.printStackTrace();
        }
        String result = ApiHelper.doPost(TuYaCloudService.TuYaOpenUrl + apiUrl,map,jsonResult);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }
    @ApiOperation("获取配网token下所有设备列表")
    @ApiImplicitParam(name="token",value = "配网的Token",required = true)
    @GetMapping("{token}/GetDevicesByToken")
    public ServerResponse getDevicesByToken(@PathVariable("token") String token){

        String apiUrl = String.format("/v1.0/devices/tokens/%s",token);
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        //Token获取后的签名
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());

        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }


    @ApiOperation("根据category获取function列表（指令集）")
    @ApiImplicitParam(name="category",value = "设备类型",required = true)
    @GetMapping("getFunctionByType")
    public ServerResponse getFunctionByType(String category){

        String apiUrl = String.format("/v1.0/functions/%s",category);
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("获取设备支持指令集")
    @ApiImplicitParam(name="deviceId",value = "设备ID",required = true)
    @GetMapping("getDeviceSupportFun")
    public ServerResponse getDeviceSupportFun(String deviceId){

        String apiUrl = String.format("/v1.0/devices/%s/functions",deviceId);
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("下发设备控制命令")
    @ApiImplicitParams({
            @ApiImplicitParam(name="deviceId",value = "设备ID",required = true)
//            @ApiImplicitParam(name="tuYaCommands",value = "命令集",required = true)
    })
    @PostMapping("sendDeviceCommandsFun")
    public ServerResponse sendDeviceCommandsFun(String deviceId,@RequestBody TuYaCommand tuYaCommands){

        String apiUrl = String.format("/v1.0/devices/%s/commands",deviceId);
        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());

        String body = JSON.toJSONString(tuYaCommands);
        String result = ApiHelper.doPost(TuYaCloudService.TuYaOpenUrl + apiUrl,map,body);

        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("根据设备ID移除设备")
    @ApiImplicitParam(name="deviceId",value = "设备ID",required = true)
    @DeleteMapping("removeDevice")
    public ServerResponse removeDevice(String deviceId){

        String apiUrl = String.format("/v1.0/devices/%s",deviceId);
        Map<String,String> map = headContent();
        String result = ApiHelper.doDelete(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("获取红外设备支持的设 备类型")
    @ApiImplicitParam(name="infraredId",value = "设备ID",required = true)
    @GetMapping("getHWCategories")
    public ServerResponse getHWCategories(String infraredId){

        String apiUrl = String.format("/v1.0/infrareds/%s/categories",infraredId);

        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("获取红外设备支持的设备品牌")
    @ApiImplicitParam(name="infraredId",value = "设备ID",required = true)
    @GetMapping("getHWBrands")
    public ServerResponse getHWBrands(String infraredId,String categoryId){

        String apiUrl = String.format("/v1.0/infrareds/%s/categories/%s/brands",infraredId,categoryId);
        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }
    @ApiOperation("获取设备品牌支持的遥控器索引")
    @ApiImplicitParam(name="infraredId",value = "设备ID",required = true)
    @GetMapping("getHWBrandsIndex")
    public ServerResponse getHWBrandsIndex(String infraredId,String categoryId,String brandId){

        String apiUrl = String.format("/v1.0/infrareds/%s/categories/%s/brands/%s",infraredId,categoryId,brandId);
        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("向红外设备添加普通遥控器")
    @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path")
    @PostMapping("{infrared_id}/addHWRemote")
    public ServerResponse addHWRemote(@RequestBody TuYaInfrared tuYaInfrared,@PathVariable String infrared_id){

        if(StringUtils.isBlank(infrared_id)||tuYaInfrared==null){
            return ServerResponse.createByErrorCodeMessage(400,"参数错误");
        }
        else{
            if(StringUtils.isBlank(tuYaInfrared.getBrand_id())||StringUtils.isBlank(tuYaInfrared.getCategory_id())||StringUtils.isBlank(tuYaInfrared.getRemote_index())){
                return ServerResponse.createByErrorCodeMessage(400,"参数错误");
            }
        }
        String apiUrl = String.format("/v1.0/infrareds/%s/normal/add-remote",infrared_id);
        Map<String,String> map = headContent();

        String json = JSON.toJSONString(tuYaInfrared);
        String result = ApiHelper.doPost(TuYaCloudService.TuYaOpenUrl + apiUrl,map,json);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("删除红外设备下的遥控器")
    @ApiImplicitParams({
        @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
        @ApiImplicitParam(name="remote_id",value = "设备ID",required = true,paramType = "path")
    })
    @DeleteMapping("{infrared_id}/delHWRemote/{remote_id}")
    public ServerResponse delHWRemote(@PathVariable String infrared_id,@PathVariable String remote_id){

        String apiUrl = String.format("/v1.0/infrareds/%s/remotes/%s",infrared_id,remote_id);
        Map<String,String> map = headContent();
        String result = ApiHelper.doDelete(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("编辑红外设备下的遥控器的名称")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path")
    })
    @PutMapping("{infrared_id}/editHWRemote")
    public ServerResponse editHWRemote(@PathVariable String infrared_id,@RequestBody TuYaInfrared tuYaInfrared){

        if(tuYaInfrared==null){
            return ServerResponse.createByErrorCodeMessage(400,"参数错误");
        }
        else{
            if(StringUtils.isBlank(tuYaInfrared.getRemote_id())||StringUtils.isBlank(tuYaInfrared.getRemote_name())){
                return ServerResponse.createByErrorCodeMessage(400,"参数错误");
            }
        }
        String apiUrl = String.format("/v1.0/infrareds/%s",infrared_id);
        Map<String,String> map = headContent();
        String json = JSON.toJSONString(tuYaInfrared);
        String result = ApiHelper.doPut(TuYaCloudService.TuYaOpenUrl + apiUrl,map,json);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("同步空调当前状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
            @ApiImplicitParam(name="remote_id",value = "设备ID",required = true,paramType = "path")
    })
    @GetMapping("{infrared_id}/getHWAirStatus/{remote_id}")
    public ServerResponse getHWAirStatus(@PathVariable String infrared_id,@PathVariable String remote_id){

        String apiUrl = String.format("/v1.0/infrareds/%s/remotes/%s/ac/status",infrared_id,remote_id);
        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("下发空调红外码")
    @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path")
    @PostMapping("{infrared_id}/addAirCode")
    public ServerResponse addAirCode(@RequestBody TuYaAirCondition tuYaAirCondition,@PathVariable String infrared_id){

        if(StringUtils.isBlank(infrared_id)||tuYaAirCondition==null){
            return ServerResponse.createByErrorCodeMessage(400,"参数错误");
        }
        else{
            if(StringUtils.isBlank(tuYaAirCondition.getRemote_index())){
                return ServerResponse.createByErrorCodeMessage(400,"参数错误");
            }
        }
        String apiUrl = String.format("/v1.0/infrareds/%s/ac/send-keys",infrared_id);
        Map<String,String> map = headContent();

        String json = JSON.toJSONString(tuYaAirCondition);
        String result = ApiHelper.doPost(TuYaCloudService.TuYaOpenUrl + apiUrl,map,json);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);

        if(tuYaResResult.getResult()!=null&&tuYaResResult.getResult().toString().equals("false")){

            tuYaResResult.setResult("控制失败");
        }else{
            tuYaResResult.setResult("控制成功");
        }
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }


    @ApiOperation("遥控器进入学习状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
            @ApiImplicitParam(name="state",value = "学习模式，true是学习模式，false非学习模式",required = true)
    })
    @GetMapping("{infrared_id}/getRemoteEnterStudy")
    public ServerResponse getRemoteEnterStudy(@PathVariable String infrared_id,@RequestParam String state){

        String apiUrl = String.format("/v1.0/infrareds/%s/learning-state?state=%s",infrared_id,state);
        Map<String,String> map = headContent();
        String result = ApiHelper.doPut(TuYaCloudService.TuYaOpenUrl + apiUrl,map,"");
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("查询到的红外码")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
            @ApiImplicitParam(name="learning_time",value = "进入学习模式的时间",required = true)
    })
    @GetMapping("{infrared_id}/getHWCode")
    public ServerResponse getHWCode(@PathVariable String infrared_id,@RequestParam String learning_time){

        String apiUrl = String.format("/v1.0/infrareds/%s/learning-codes?learning_time=%s",infrared_id,learning_time);
        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("获取学习到的红外码（根据遥控器设备）")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
            @ApiImplicitParam(name="remote_id",value = "遥控器ID",required = true,paramType = "path")
    })
    @GetMapping("{infrared_id}/getHWCode/{remote_id}")
    public ServerResponse getHWCodeByRemoteId(@PathVariable String infrared_id,@PathVariable String remote_id){

        String apiUrl = String.format("/v1.0/infrareds/%s/remotes/%s/learning-codes",infrared_id,remote_id);
        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("退出学习模式")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path")
    })
    @GetMapping("{infrared_id}/hwQuitSyudy")
    public ServerResponse hwQuitSyudy(@PathVariable String infrared_id){

        String apiUrl = String.format("/v1.0/infrareds/%s/learning-quit",infrared_id);
        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("删除学习到的红外码")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
            @ApiImplicitParam(name="learn_id",value = "学习到的红外ID",required = true,paramType = "path")
    })
    @DeleteMapping("{infrared_id}/deteleLearnCode/{learn_id}")
    public ServerResponse deteleLearnCode(@PathVariable String infrared_id,@PathVariable String learn_id){

        String apiUrl = String.format("/v1.0/infrareds/%s/learning-codes/%s",infrared_id,learn_id);
        Map<String,String> map = headContent();
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("保存学习到红外码")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path")
    })
    @PostMapping("{infrared_id}/saveLearnCode")
    public ServerResponse saveLearnCode(@PathVariable String infrared_id,@RequestBody TuYaInfrared tuYaInfrared){

        if(tuYaInfrared==null){
            return ServerResponse.createByErrorCodeMessage(400,"参数错误");
        }

        String apiUrl = String.format("/v1.0/infrareds/%s/learning-codes",infrared_id);
        Map<String,String> map = headContent();
        String json = JSON.toJSONString(tuYaInfrared);
        String result = ApiHelper.doPost(TuYaCloudService.TuYaOpenUrl + apiUrl,map,json);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    @ApiOperation("更新学习到红外码")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
            @ApiImplicitParam(name="remote_index",value = "遥控器索引",required = true,paramType = "path")
    })
    @PutMapping("{infrared_id}/editLearnCode/{remote_index}")
    public ServerResponse saveLearnCode(@PathVariable String infrared_id,@PathVariable String remote_index,@RequestBody TuYaInfrared tuYaInfrared){

        if(tuYaInfrared==null){
            return ServerResponse.createByErrorCodeMessage(400,"参数错误");
        }

        String apiUrl = String.format("/v1.0/infrareds/%s/learning-remotes/%s",infrared_id);
        Map<String,String> map = headContent();
        String json = JSON.toJSONString(tuYaInfrared);
        String result = ApiHelper.doPut(TuYaCloudService.TuYaOpenUrl + apiUrl,map,json);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }
    @ApiOperation("下发到红外到的红外码")
    @ApiImplicitParams({
            @ApiImplicitParam(name="infrared_id",value = "设备ID",required = true,paramType = "path"),
            @ApiImplicitParam(name="remote_id",value = "遥控器设备ID（没有填TUYA）",required = true,paramType = "path"),
            @ApiImplicitParam(name="code",value = "学习到的红外码",required = true)
    })
    @GetMapping("{infrared_id}/getRemoteEnterStudy/{remote_id}")
    public ServerResponse getRemoteEnterStudy(@PathVariable String infrared_id,@PathVariable String remote_id,@RequestParam String code){

        String apiUrl = String.format("/v1.0/infrareds/%s/remote/%s/learning-codes",infrared_id,remote_id);
        Map<String,String> map = headContent();
        HWCode hwCode = new HWCode();
        hwCode.setCode(code);
        String jsonResult = JSON.toJSONString(hwCode);
        String result = ApiHelper.doPost(TuYaCloudService.TuYaOpenUrl + apiUrl,map,jsonResult);
        TuYaResResult tuYaResResult = JSON.parseObject(result,TuYaResResult.class);
        return ResultUtils.getInstance().returnResultTuYa(tuYaResResult);
    }

    /**
     * 请求头的参数
     * @return
     */
    private Map<String,String> headContent(){

        Long t = System.currentTimeMillis();
        String access_Token = CacheHelper.getTuYaToken();
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+access_Token+t,TuYaCloudService.Secret);

        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("access_token",access_Token);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());

        return map;
    }
}
