package com.sincre.springboot.controller.Yinshi;

import com.alibaba.fastjson.JSON;
import com.sincre.springboot.ApiModel.YinShiResResult;
import com.sincre.springboot.ApiPlatform.YinShiServiceConfig;
import com.sincre.springboot.common.ServerResponse;
import com.sincre.springboot.utils.ApiHelper;
import com.sincre.springboot.utils.ResultUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/YinShiDevice")
@Api(tags = "设备管理部分接口—楼宇设备和客流统计及设备升级")
public class DeviceControl {


    @ApiOperation(value = "获取通话状态")
    @ApiImplicitParam(name="deviceSerial",value = "设备序列号",required = true)
    @GetMapping("getDeviceCallStatus")
    public ServerResponse getDeviceCallStatus(@RequestParam String deviceSerial) {

        String url = YinShiServiceConfig.HostUrl + "lapp/building/device/call/status";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);

        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }

    @ApiOperation(value = "该接口用于楼宇可视对讲设备远程开锁")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceSerial", value = "设备序列号", required = true),
            @ApiImplicitParam(name = "type",value = "开锁类型，1-监视开锁；2-通话开锁",required = true),
            @ApiImplicitParam(name = "channel",value = "监视通道，1-主门口机，2-1号从门口机，3-2号从门口机，不能为1-9之外的数字",required = true),
            @ApiImplicitParam(name = "lockId",value = "锁ID，0本地锁，1外接锁")
    })
    @GetMapping("deviceUnlock")
    public ServerResponse deviceUnlock(@RequestParam String deviceSerial,@RequestParam Integer type,@RequestParam Integer channel,@RequestParam(defaultValue = "-1") Integer lockId) {

        String url = YinShiServiceConfig.HostUrl + "lapp/building/device/unlock";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        map.put("type",type);
        map.put("channel",channel);

        System.out.println(lockId);
        if(lockId==-1) {
            map.put("lockId", lockId);
        }
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);

        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }

    @ApiOperation(value = "该接口用于楼宇可视对讲设备通话操作")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceSerial", value = "设备序列号", required = true),
            @ApiImplicitParam(name = "type",value = "操作类型：2-接听，3-拒接， 4-响铃超时，5-挂断，6-被叫正在通话中",required = true)
    })
    @GetMapping("setDeviceCall")
    public ServerResponse setDeviceCall(@RequestParam String deviceSerial,@RequestParam Integer type) {

        String url = YinShiServiceConfig.HostUrl + "lapp/building/device/call";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        map.put("type",type);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);

        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }

    @ApiOperation(value = "该接口用于获取楼宇可视对讲设备主叫信息。")
    @ApiImplicitParam(name="deviceSerial",value = "设备序列号",required = true)
    @GetMapping("getDeviceCallInfo")
    public ServerResponse getDeviceCallInfo(@RequestParam String deviceSerial) {

        String url = YinShiServiceConfig.HostUrl + "lapp/building/device/dialing/get";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);

        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }
    @ApiOperation(value = "该接口用于获取楼宇可视对讲设备门口机列表。")
    @ApiImplicitParam(name="deviceSerial",value = "设备序列号",required = true)
    @GetMapping("getDeviceCallList")
    public ServerResponse getDeviceCallList(@RequestParam String deviceSerial) {

        String url = YinShiServiceConfig.HostUrl + "lapp/building/device/list";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);

        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }



    @ApiOperation(value = "获取客流统计开关状态。")
    @ApiImplicitParam(name="deviceSerial",value = "设备序列号",required = true)
    @GetMapping("getCustomFlowCount")
    public ServerResponse getCustomFlowCount(@RequestParam String deviceSerial) {

        String url = YinShiServiceConfig.HostUrl + "lapp/passengerflow/switch/status";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);

        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }

    @ApiOperation(value = "查询用户下指定设备的版本信息。")
    @ApiImplicitParam(name="deviceSerial",value = "设备序列号",required = true)
    @GetMapping("getDeviceVersionInfo")
    public ServerResponse getDeviceVersionInfo(@RequestParam String deviceSerial) {

        String url = YinShiServiceConfig.HostUrl + "lapp/device/version/info";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);
        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }

    @ApiOperation(value = "升级设备固件至最新版本。")
    @ApiImplicitParam(name="deviceSerial",value = "设备序列号",required = true)
    @GetMapping("deviceUpgrade")
    public ServerResponse deviceUpgrade(@RequestParam String deviceSerial) {

        String url = YinShiServiceConfig.HostUrl + "lapp/device/upgrade";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);
        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }


    @ApiOperation(value = "查询用户下指定设备的升级状态，包括升级进度。")
    @ApiImplicitParam(name="deviceSerial",value = "设备序列号",required = true)
    @GetMapping("deviceUpgradeStatus")
    public ServerResponse deviceUpgradeStatus(@RequestParam String deviceSerial) {

        String url = YinShiServiceConfig.HostUrl + "lapp/device/upgrade/status";
        Map<String, Object> map = new HashMap<>();
        map.put("accessToken", YinShiServiceConfig.AccessToken);
        map.put("deviceSerial", deviceSerial);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);
        YinShiResResult yinShiResResult = JSON.parseObject(result,YinShiResResult.class);

        return ResultUtils.getInstance().returnResultYingshi(yinShiResResult);
    }



}
