package com.sincre.springboot.utils;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.*;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.*;

public class ApiHelper {

    public static String doGet(String url,Map<String, String> headerParamMap) {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse response = null;
        String result = "";
        try {
            // 通过址默认配置创建一个httpClient实例
            httpClient = HttpClients.createDefault();
            // 创建httpGet远程连接实例
            HttpGet httpGet = new HttpGet(url);
            // 设置请求头信息，
            for (Map.Entry<String, String> entry : headerParamMap.entrySet()) {
                httpGet.setHeader(entry.getKey(),entry.getValue());
            }

            // 为httpGet实例设置配置
            httpGet.setConfig(setRequestConfig());
            // 执行get请求得到返回对象
            response = httpClient.execute(httpGet);
            // 通过返回对象获取返回数据
            HttpEntity entity = response.getEntity();
            // 通过EntityUtils中的toString方法将结果转换为字符串
            result = EntityUtils.toString(entity);
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            if (null != response) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != httpClient) {
                try {
                    httpClient.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return result;
    }



    public static String doDelete(String url,Map<String, String> headerParamMap) {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse response = null;
        String result = "";
        try {
            // 通过址默认配置创建一个httpClient实例
            httpClient = HttpClients.createDefault();
            // 创建HttpDelete远程连接实例
            HttpDelete httpDelete = new HttpDelete(url);
            // 设置请求头信息，
            for (Map.Entry<String, String> entry : headerParamMap.entrySet()) {
                httpDelete.setHeader(entry.getKey(),entry.getValue());
            }
            // 为httpGet实例设置配置
            httpDelete.setConfig(setRequestConfig());
            // 执行get请求得到返回对象
            response = httpClient.execute(httpDelete);
            // 通过返回对象获取返回数据
            HttpEntity entity = response.getEntity();
            // 通过EntityUtils中的toString方法将结果转换为字符串
            result = EntityUtils.toString(entity);
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            if (null != response) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != httpClient) {
                try {
                    httpClient.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return result;
    }

    /**
     * 表单格式的post请求
     *
     * @param url            接口地址
     * @param headerParamMap 设置请求头
     * @param paramMap       设置请求值，表单格式的请求值
     * @return
     */
    public static String doPost(String url, Map<String, String> headerParamMap, Map<String, Object> paramMap) {
        CloseableHttpClient httpClient;
        String result;
        // 创建httpClient实例
        httpClient = HttpClients.createDefault();
        // 创建httpPost远程连接实例
        HttpPost httpPost = new HttpPost(url);
        // 设置请求头
        httpPost.addHeader("Content-Type", "application/x-www-form-urlencoded");
        for (Map.Entry<String, String> entry : headerParamMap.entrySet()) {
            httpPost.setHeader(entry.getKey(), entry.getValue());
        }
        // 为httpPost实例设置配置
        httpPost.setConfig(setRequestConfig());

        // 封装post请求参数
        if (null != paramMap && paramMap.size() > 0) {
            List<NameValuePair> nvps = new ArrayList<NameValuePair>();
            // 通过map集成entrySet方法获取entity
            Set<Map.Entry<String, Object>> entrySet = paramMap.entrySet();
            // 循环遍历，获取迭代器
            Iterator<Map.Entry<String, Object>> iterator = entrySet.iterator();
            while (iterator.hasNext()) {
                Map.Entry<String, Object> mapEntry = iterator.next();
                nvps.add(new BasicNameValuePair(mapEntry.getKey(), mapEntry.getValue().toString()));
            }

            // 为httpPost设置封装好的请求参数
            try {
                UrlEncodedFormEntity entity = new UrlEncodedFormEntity(nvps, "UTF-8");
                httpPost.setEntity(entity);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }
        result = closeHttpAndResult(httpClient, httpPost,null);
        return result;
    }
    /**
     * post请求，内容体比较灵活，任意字符串格式的内容
     * @param url
     * @param headerParamMap
     * @param jsonParam
     * @return
     */
    public static String doPost(String url, Map<String, String> headerParamMap, String jsonParam) {
        CloseableHttpClient httpClient;
        String result;
        // 创建httpClient实例
        httpClient = HttpClients.createDefault();
        // 创建httpPost远程连接实例
        HttpPost httpPost = new HttpPost(url);
        // 设置请求头
        httpPost.addHeader("Content-Type", "application/json");
        for (Map.Entry<String, String> entry : headerParamMap.entrySet()) {
            httpPost.setHeader(entry.getKey(),entry.getValue());
        }
        // 为httpPost实例设置配置
        httpPost.setConfig(setRequestConfig());
//        try {
        StringEntity stringEntity = new StringEntity(jsonParam, ContentType.create("application/json","UTF-8"));
//            stringEntity.setContentType("application/json");
//            stringEntity.setContentEncoding("gzip");//请求头的编码格式
        httpPost.setEntity(stringEntity);
//        }catch (UnsupportedEncodingException e){
//            e.printStackTrace();
//        }
        result = closeHttpAndResult(httpClient,httpPost,null);
        return result;
    }

    public static String doPut(String url, Map<String, String> headerParamMap, String jsonParam) {
        CloseableHttpClient httpClient;
        String result;
        // 创建httpClient实例
        httpClient = HttpClients.createDefault();
        // 创建httpPost远程连接实例
        HttpPut httpPut= new HttpPut(url);
        // 设置请求头
        httpPut.addHeader("Content-Type", "application/json");
        for (Map.Entry<String, String> entry : headerParamMap.entrySet()) {
            httpPut.setHeader(entry.getKey(),entry.getValue());
        }
        // 为httpPut实例设置配置
        httpPut.setConfig(setRequestConfig());

        StringEntity stringEntity = new StringEntity(jsonParam, ContentType.create("application/json","UTF-8"));

        httpPut.setEntity(stringEntity);

        result = closeHttpAndResult(httpClient,null,httpPut);
        return result;
    }


    private static String closeHttpAndResult(CloseableHttpClient httpClient,HttpPost httpPost,HttpPut httpPut){
        CloseableHttpResponse httpResponse = null;
        String result = null;
        try {
            // httpClient对象执行post或者是put请求,并返回响应参数对象
            if(httpPost != null) {
                httpResponse = httpClient.execute(httpPost);
            }else {
                httpResponse = httpClient.execute(httpPut);
            }
            // 从响应对象中获取响应内容
            HttpEntity entity = httpResponse.getEntity();
            result = EntityUtils.toString(entity);
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            if (null != httpResponse) {
                try {
                    httpResponse.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != httpClient) {
                try {
                    httpClient.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return result;
    }


    /**
     * 设置配置请求参数
     * @return
     */
    private static RequestConfig setRequestConfig(){
        RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(35000)// 连接主机服务超时时间
                .setConnectionRequestTimeout(35000)// 请求超时时间
                .setSocketTimeout(60000)// 数据读取超时时间
                .build();

        return  requestConfig;
    }
}

