package com.sincre.springboot.utils;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sincre.springboot.ApiModel.TuYaResResult;
import com.sincre.springboot.ApiModel.TuYaResTokenObj;
import com.sincre.springboot.ApiPlatform.BaiduYunServiceConfig;
import com.sincre.springboot.ApiPlatform.TuYaCloudService;
import com.sincre.springboot.ApiPlatform.YinShiServiceConfig;
import com.sincre.springboot.common.EhcacheUtil;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;
import org.json.JSONObject;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;
import java.util.Map;
public class CacheHelper {

    static EhcacheUtil ehcacheUtil = new EhcacheUtil();
    static String TuYaTokenKey = "TuYa_Token";
    static String TuYaRefreshTokenKey = "TuYa_Re_Token";


    private static void setTuYaToken(){

        String apiUrl = "/v1.0/token?grant_type=1";
        Long t = System.currentTimeMillis();
        //Token获取前的签名
        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);
        System.out.println(result);
        TuYaResResult<TuYaResTokenObj> resResult ;
        try{
            ObjectMapper objectMapper = new ObjectMapper();
            resResult = objectMapper.readValue(result,new TypeReference<TuYaResResult<TuYaResTokenObj>>(){});

            String access_Token = resResult.getResult().getAccess_token(); //存储第一次获取的Token
            String refresh_Token = resResult.getResult().getRefresh_token();
               // 第一次缓存数据的时候，不刷新，后期刷新
            if(StringUtils.isNotBlank(ehcacheUtil.get(TuYaTokenKey))){
                TuYaRefresh(resResult.getResult());
            }else {
                ehcacheUtil.setex(TuYaTokenKey, access_Token, resResult.getResult().getExpire_time()); //  储存二个小时
                ehcacheUtil.setex(TuYaRefreshTokenKey,refresh_Token,60 * 60); //  储存一个小时
                System.out.println("获取Token："+ access_Token);
                System.out.println("刷新Token："+ refresh_Token);

            }
        } catch(Exception ex){
            System.out.println(ex.getMessage());
        }
    }

    /**
     * 获取涂鸦Api 访问的Token
     * @return
     */
    public static String getTuYaToken(){

        if(StringUtils.isBlank(ehcacheUtil.get(TuYaRefreshTokenKey))){
            setTuYaToken();
        }

        return  ehcacheUtil.get(TuYaTokenKey);
    }

    private static void TuYaRefresh(TuYaResTokenObj tuYaResTokenObj){

        String apiUrl = "/v1.0/token/"+tuYaResTokenObj.getRefresh_token();
        Long t = System.currentTimeMillis();

        String sign = TuYaCloudService.createSign(TuYaCloudService.ClientId+t,TuYaCloudService.Secret);
        Map<String,String> map = new HashMap();
        map.put("client_id", TuYaCloudService.ClientId);
        map.put("sign",sign);
        map.put("sign_method",TuYaCloudService.Sign_method);
        map.put("t",t.toString());
        String result = ApiHelper.doGet(TuYaCloudService.TuYaOpenUrl + apiUrl,map);

        TuYaResResult<TuYaResTokenObj> resResult;
        try{
            ObjectMapper objectMapper = new ObjectMapper();
            TypeReference<TuYaResResult<TuYaResTokenObj>> reference = new TypeReference<TuYaResResult<TuYaResTokenObj>>(){};
            resResult = objectMapper.readValue(result,reference);
            String re_access_Token = resResult.getResult().getAccess_token(); //重新刷新后获取的Token
            String refresh_Token = resResult.getResult().getRefresh_token();
            System.out.println("刷新后的Token："+re_access_Token);
            ehcacheUtil.setex(TuYaTokenKey,re_access_Token,resResult.getResult().getExpire_time()); //  储存两个小时
            ehcacheUtil.setex(TuYaRefreshTokenKey,refresh_Token,60 * 60); //储存一个小时
        } catch(Exception ex){
            System.out.println(ex.getMessage());
        }
    }

    /**
     * 获取萤石云token
     * @return
     */
    public static String getYingshiYunToken(){
        return ehcacheUtil.get(YinShiServiceConfig.accessToken);
    }

    /**
     * 设置萤石云token
     * @param token
     * @param time
     */
    public static void putYingshiYunToken(String token,int time){
        ehcacheUtil.setex(YinShiServiceConfig.accessToken,token,time);
    }

    /**
     * 获得萤石管理权限的Token
     */
    public static void GetYinShiToken() {
        String appKey = YinShiServiceConfig.appKey;
        String appSecret = YinShiServiceConfig.appSecret;

        String url = YinShiServiceConfig.HostUrl + "lapp/token/get";
        Map<String, Object> map = new HashMap<>();

        map.put("appKey", appKey);
        map.put("appSecret", appSecret);
        String result = ApiHelper.doPost(url, new HashMap<String, String>(), map);
        JSONObject jsonObject = new JSONObject(result);
        JSONObject data = jsonObject.optJSONObject("data");
        YinShiServiceConfig.AccessToken = data.optString("accessToken");
    }

    /**
     * 获取权限token
     *
     * @return 返回示例：
     * {
     * "access_token": "24.460da4889caad24cccdb1fea17221975.2592000.1491995545.282335-1234567",
     * "expires_in": 2592000
     * }
     */
    public static String getBaiDuToken() {
        // 官网获取的 API Key 更新为你注册的
        String clientId = BaiduYunServiceConfig.client_id;
        // 官网获取的 Secret Key 更新为你注册的
        String clientSecret = BaiduYunServiceConfig.client_secret;
        return getAuth(clientId, clientSecret);
    }

    /**
     * 获取API访问token
     * 该token有一定的有效期，需要自行管理，当失效时需重新获取.
     *
     * @param ak - 百度云官网获取的 API Key
     * @param sk - 百度云官网获取的 Securet Key
     * @return assess_token 示例：
     * "24.460da4889caad24cccdb1fea17221975.2592000.1491995545.282335-1234567"
     */
    public static String getAuth(String ak, String sk) {

        // 获取token地址
        String authHost = "https://aip.baidubce.com/oauth/2.0/token?";
        String getAccessTokenUrl = authHost
                // 1. grant_type为固定参数
                + "grant_type=client_credentials"
                // 2. 官网获取的 API Key
                + "&client_id=" + ak
                // 3. 官网获取的 Secret Key
                + "&client_secret=" + sk;
        try {
            URL realUrl = new URL(getAccessTokenUrl);
            // 打开和URL之间的连接
            HttpURLConnection connection = (HttpURLConnection) realUrl.openConnection();
            connection.setRequestMethod("GET");
            connection.connect();
            // 获取所有响应头字段
            Map<String, List<String>> map = connection.getHeaderFields();
            // 遍历所有的响应头字段
            for (String key : map.keySet()) {
                System.err.println(key + "--->" + map.get(key));
            }
            // 定义 BufferedReader输入流来读取URL的响应
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String result = "";
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
            /**
             * 返回结果示例
             */
            System.err.println("result:" + result);
            JSONObject jsonObject = new JSONObject(result);
            String access_token = jsonObject.getString("access_token");
            return access_token;
        } catch (Exception e) {
            System.err.printf("获取token失败！");
            e.printStackTrace(System.err);
        }
        return null;
    }
}
